# @param middleware_home_dir [String] The middleware home directory, default is fetched from hiera.
# @param weblogic_home_dir [String] The WebLogic home directory, default is fetched from hiera.
# @param jdk_home_dir [String] The JDK home directory, default is fetched from hiera.
# @param wls_domains_dir [String, Undef] The WebLogic domains directory, default is fetched from hiera or undef.
# @param domain_name [String, Undef] The domain name, default is fetched from hiera or undef.
# @param adapter_name [String, Undef] The name of the adapter, default is undef.
# @param adapter_path [String, Undef] The path to the adapter, default is undef.
# @param adapter_plan_dir [String, Undef] The directory for the adapter plan, default is undef.
# @param adapter_plan [String, Undef] The adapter plan, default is undef.
# @param adapter_entry [String, Undef] The adapter entry, default is undef.
# @param adminserver_address [String] The address of the admin server, default is 'localhost'.
# @param adminserver_port [Integer] The port of the admin server, default is 7001.
# @param user_config_file [String, Undef] The user configuration file, default is fetched from hiera or undef.
# @param user_key_file [String, Undef] The user key file, default is fetched from hiera or undef.
# @param weblogic_user [String] The WebLogic user, default is 'weblogic'.
# @param weblogic_password [String, Undef] The WebLogic password, default is fetched from hiera or undef.
# @param os_user [String] The operating system user, default is fetched from hiera.
# @param os_group [String] The operating system group, default is fetched from hiera.
# @param download_dir [String] The directory for downloads, default is fetched from hiera.
# @param logoutput [Boolean] Whether to log output, default is false.
# @param adapter_properties [Hash] Properties for the adapter, default is an empty hash.
# @param pool_configuration [Hash] Configuration for the connection pool, default is an empty hash.
define wls_install::resourceadapter (
  String[1]           $middleware_home_dir    = hiera('wls_middleware_home_dir'), # /opt/oracle/middleware11gR1
  String[1]           $weblogic_home_dir      = hiera('wls_weblogic_home_dir'),
  String[1]           $jdk_home_dir           = hiera('wls_jdk_home_dir'), # /usr/java/jdk1.7.0_45
  String[1]           $wls_domains_dir        = hiera('wls_domains_dir'           , undef),
  String[1]           $domain_name            = hiera('domain_name'               , undef),
  String[1]           $adminserver_address    = hiera('domain_adminserver_address', 'localhost'),
  Integer             $adminserver_port       = hiera('domain_adminserver_port'   , 7001),
  String[1]           $weblogic_user          = hiera('wls_weblogic_user'         , 'weblogic'),
  Easy_type::Password $weblogic_password      = hiera('domain_wls_password'       , undef),
  String[1]           $os_user                = hiera('wls_os_user'), # oracle
  String[1]           $os_group               = hiera('wls_os_group'), # dba
  String[1]           $download_dir           = hiera('wls_download_dir'), # /data/install
  Optional[String[1]] $user_config_file       = hiera('domain_user_config_file'   , undef),
  Optional[String[1]] $user_key_file          = hiera('domain_user_key_file'      , undef),
  Hash                $adapter_properties     = {},
  Hash                $pool_configuration     = {},
  Optional[String[1]] $adapter_name           = undef,
  Optional[String[1]] $adapter_path           = undef,
  Optional[String[1]] $adapter_plan_dir       = undef,
  Optional[String[1]] $adapter_plan           = undef,
  Optional[String[1]] $adapter_entry          = undef,
  Variant[
    Boolean,
    Enum[
      'on_failure',
    ]
  ]                   $logoutput              = lookup({ name => 'logoutput', default_value => 'on_failure' }),
) {
  $sanetized_title = easy_type::sanetize($title)
  if ( $wls_domains_dir == undef or $wls_domains_dir == '') {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  } else {
    $domains_dir =  $wls_domains_dir
  }

  $domain_dir = "${domains_dir}/${domain_name}"

  easy_type::debug_evaluation() # Show local variable on extended debug

  # if these params are empty always continue
  if $domain_name == undef or $adapter_name == undef or $adapter_plan_dir == undef or $adapter_plan == undef {
    fail('domain, adapter_name,adapter_plan_dir or adapter_plan is nill')
  } else {
    # check if the object already exists on the weblogic domain
    $found = wls_install::artifact_exists($domain_dir,'resource',$adapter_name,"${adapter_plan_dir}/${adapter_plan}" )
    if $found == undef {
      $continue_plan = true
      debug "wls_install::resourceadapter ${title} continue cause nill"
    } else {
      if ( $found ) {
        $continue_plan = false
      } else {
        debug "wls_install::resourceadapter ${title} continue, does not exists"
        $continue_plan = true
      }
    }
  }

  # if these params are empty always continue
  if $domain_name == undef or $adapter_name == undef or $adapter_entry == undef {
    fail('domain, adapter_name or adapter_entry is nill')
  } else {
    # check if the object already exists on the weblogic domain
    $found_entry = wls_install::artifact_exists($domain_dir ,'resource_entry', $adapter_name, $adapter_entry )
    if $found_entry == undef {
      $continue_entry = true
      debug "wls_install::resourceadapter entry ${adapter_entry} ${title} continue cause nill"
    } else {
      if ( $found_entry ) {
        $continue_entry = false
      } else {
        debug "wls_install::resourceadapter entry ${adapter_entry} ${title} continue, does not exists"
        $continue_entry = true
      }
    }
  }

  $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

  if $user_config_file != undef {
    $credentials    =   " -userconfigfile ${user_config_file} -userkeyfile ${user_key_file}"
    $use_store_config = true
  } elsif $weblogic_user != undef {
    $credentials =   " -user ${weblogic_user} -password '${weblogic_password.unwrap}'"
    $use_store_config = false
  } else {
    fail('user_config_file wlsUser parameter is empty')
  }

  case $adapter_name {
    'JmsAdapter': {
      $connection_factory_interface='oracle.tip.adapter.jms.IJmsConnectionFactory'
      if !defined(File["${adapter_plan_dir}/${adapter_plan}"]) {
        file { "${adapter_plan_dir}/${adapter_plan}":
          ensure  => file,
          mode    => '0744',
          replace => false,
          owner   => $os_user,
          group   => $os_group,
          backup  => false,
          path    => "${adapter_plan_dir}/${adapter_plan}",
          content => epp('wls_install/adapter_plans/Plan_JMS.xml.epp', { 'adapter_plan_dir' => $adapter_plan_dir }),
        }
      }
    }
    'DbAdapter' : {
      $connection_factory_interface='javax.resource.cci.ConnectionFactory'
      if !defined(File["${adapter_plan_dir}/${adapter_plan}"]) {
        file { "${adapter_plan_dir}/${adapter_plan}":
          ensure  => file,
          mode    => '0744',
          replace => false,
          owner   => $os_user,
          group   => $os_group,
          backup  => false,
          path    => "${adapter_plan_dir}/${adapter_plan}",
          content => epp('wls_install/adapter_plans/Plan_DB.xml.epp', { 'adapter_plan_dir' => $adapter_plan_dir }),
        }
      }
    }
    'AqAdapter' : {
      $connection_factory_interface='javax.resource.cci.ConnectionFactory'
      if !defined(File["${adapter_plan_dir}/${adapter_plan}"]) {
        file { "${adapter_plan_dir}/${adapter_plan}":
          ensure  => file,
          mode    => '0744',
          replace => false,
          owner   => $os_user,
          group   => $os_group,
          backup  => false,
          path    => "${adapter_plan_dir}/${adapter_plan}",
          content => epp('wls_install/adapter_plans/Plan_AQ.xml.epp', { 'adapter_plan_dir' => $adapter_plan_dir }),
        }
      }
    }
    'FtpAdapter' : {
      $connection_factory_interface='javax.resource.cci.ConnectionFactory'
      if !defined(File["${adapter_plan_dir}/${adapter_plan}"]) {
        file { "${adapter_plan_dir}/${adapter_plan}":
          ensure  => file,
          mode    => '0744',
          replace => false,
          owner   => $os_user,
          group   => $os_group,
          backup  => false,
          path    => "${adapter_plan_dir}/${adapter_plan}",
          content => epp('wls_install/adapter_plans/Plan_FTP.xml.epp', { 'adapter_plan_dir' => $adapter_plan_dir }),
        }
      }
    }
    'FileAdapter' : {
      $connection_factory_interface='javax.resource.cci.ConnectionFactory'
      if !defined(File["${adapter_plan_dir}/${adapter_plan}"]) {
        file { "${adapter_plan_dir}/${adapter_plan}":
          ensure  => file,
          mode    => '0744',
          replace => false,
          owner   => $os_user,
          group   => $os_group,
          backup  => false,
          path    => "${adapter_plan_dir}/${adapter_plan}",
          content => epp('wls_install/adapter_plans/Plan_File.xml.epp', { 'adapter_plan_dir' => $adapter_plan_dir }),
        }
      }
    }
    'MQSeriesAdapter' : {
      $connection_factory_interface='javax.resource.cci.ConnectionFactory'
      if !defined(File["${adapter_plan_dir}/${adapter_plan}"]) {
        file { "${adapter_plan_dir}/${adapter_plan}":
          ensure  => file,
          mode    => '0744',
          replace => false,
          owner   => $os_user,
          group   => $os_group,
          backup  => false,
          path    => "${adapter_plan_dir}/${adapter_plan}",
          content => epp('wls_install/adapter_plans/Plan_MQSeries.xml.epp', { 'adapter_plan_dir' => $adapter_plan_dir }),
        }
      }
    }
    default: {
      fail("adapter_name ${adapter_name} is unknown, choose FileAdapter, DbAdapter, JmsAdapter, FtpAdapter, MQSeriesAdapter or AqAdapter ")
    }
  }

  # lets make the a new plan for this adapter
  if ( $continue_plan ) {
    case $facts['kernel'] {
      'Linux': {
        $java_statement = 'java weblogic.Deployer'
      }
      'SunOS': {
        $java_statement = 'java -d64 weblogic.Deployer'
      }
      default: {
        fail("kernel ${facts['kernel']} is unknown, choose Linux or SunOS")
      }
    }

    case $facts['kernel'] {
      'Linux','SunOS': {
        # deploy the plan and update the adapter
        exec { "exec deployer adapter plan ${title}":
          command     => "${java_statement} -adminurl t3://${adminserver_address}:${adminserver_port} ${credentials} -update -name ${adapter_name} -plan ${adapter_plan_dir}/${adapter_plan}",
          environment => ["CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar", "JAVA_HOME=${jdk_home_dir}"],
          cwd         => $download_dir,
          path        => $exec_path,
          user        => $os_user,
          group       => $os_group,
          logoutput   => $logoutput,
          require     => File["${adapter_plan_dir}/${adapter_plan}"],
          before      => Exec["exec create resource adapter entry ${title}"],
        }
      }
      default: {
        fail("kernel ${facts['kernel']} is unknown, choose Linux or SunOS")
      }
    }
  }

  # after deployment of the plan we can add a new entry to the adapter
  if ( $continue_entry ) {
    $java_command_plan = "${middleware_home_dir}/oracle_common/common/bin/wlst.sh"

    file { "${download_dir}/${title}redeployResourceAdapter.py":
      ensure  => file,
      mode    => '0744',
      owner   => $os_user,
      group   => $os_group,
      backup  => false,
      path    => "${download_dir}/${sanetized_title}redeployResourceAdapter.py",
      content => epp('wls_install/adapter_plans/redeployResourceAdapter.py.epp', {
          'weblogic_user'       => $weblogic_user,
          'adminserver_address' => $adminserver_address,
          'adminserver_port'    => $adminserver_port,
          'adapter_name'        => $adapter_name,
          'adapter_plan_dir'    => $adapter_plan_dir,
          'adapter_plan'        => $adapter_plan,
          'use_store_config'    => $use_store_config,
          'user_config_file'    => $user_config_file,
          'user_key_file'       => $user_key_file
      }),
      before  => Exec["exec redeploy adapter plan ${title}"],
    }
    cleanup { "${download_dir}/${sanetized_title}redeployResourceAdapter.py": }

    file { "${download_dir}/${sanetized_title}createResourceAdapterEntry.py":
      ensure  => file,
      mode    => '0744',
      owner   => $os_user,
      group   => $os_group,
      backup  => false,
      path    => "${download_dir}/${sanetized_title}createResourceAdapterEntry.py",
      content => epp('wls_install/adapter_plans/createResourceAdapterEntry.py.epp', {
          'weblogic_user'                => $weblogic_user,
          'adminserver_address'          => $adminserver_address,
          'adminserver_port'             => $adminserver_port,
          'adapter_entry'                => $adapter_entry,
          'adapter_properties'           => $adapter_properties,
          'pool_configuration'           => $pool_configuration,
          'connection_factory_interface' => $connection_factory_interface,
          'adapter_name'                 => $adapter_name,
          'adapter_path'                 => $adapter_path,
          'adapter_plan_dir'             => $adapter_plan_dir,
          'adapter_plan'                 => $adapter_plan,
          'use_store_config'             => $use_store_config,
          'user_config_file'             => $user_config_file,
          'user_key_file'                => $user_key_file,
      }),
      before  => Exec["exec create resource adapter entry ${title}"],
    }
    cleanup { "${download_dir}/${sanetized_title}createResourceAdapterEntry.py": }

    case $facts['kernel'] {
      'Linux','SunOS': {
        # deploy the plan and update the adapter
        exec { "exec create resource adapter entry ${title}":
          command     => "${java_command_plan} ${download_dir}/${sanetized_title}createResourceAdapterEntry.py '${weblogic_password.unwrap}'",
          environment => ["CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar", "JAVA_HOME=${jdk_home_dir}"],
          cwd         => $download_dir,
          path        => $exec_path,
          user        => $os_user,
          group       => $os_group,
          logoutput   => $logoutput,
          require     => File["${adapter_plan_dir}/${adapter_plan}"],
        }

        # deploy the plan and update the adapter
        exec { "exec redeploy adapter plan ${title}":
          command     => "${java_command_plan} ${download_dir}/${sanetized_title}redeployResourceAdapter.py '${weblogic_password.unwrap}'",
          environment => ["CLASSPATH=${weblogic_home_dir}/server/lib/weblogic.jar", "JAVA_HOME=${jdk_home_dir}"],
          cwd         => $download_dir,
          path        => $exec_path,
          user        => $os_user,
          group       => $os_group,
          logoutput   => $logoutput,
          require     => [Exec["exec create resource adapter entry ${title}"],File["${adapter_plan_dir}/${adapter_plan}"],],
        }
      }
      default: {
        fail("kernel ${facts['kernel']} is unknown, choose Linux or SunOS")
      }
    }
  }
}
